#include <math.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define WIN_WIDTH   600
#define WIN_HEIGHT  600
#define V_SCALE      60
#define H_SCALE      60 
#define NP          300
#define NL          500

/* structure */
struct COORD {
  double x;
  double y;
  double z;
};	

struct CONNECT {
  int bgn;
  int end;
};

struct COORD    coord [ NP ];
struct CONNECT   cnct [ NL ];

int axis = False;

/* functions */

double torad ( double t ) {
  return ( t / 180 * M_PI ) ;
}

int mulmat ( double *x, double *y, double th ) {
  double xt,yt,si,co;
  si = sin ( th ) ;
  co = cos ( th ) ;
  xt = *x *co + *y *si;
  yt = - *x *si + *y *co;
  *x = xt;  *y = yt;
}

int trans ( double x, double y, double z, int *px, int *py, double th, double ph ) {
  mulmat ( &x, &y, th ) ;
  mulmat ( &x, &z, ph ) ;
  *px = WIN_WIDTH / 2 + (int)( y * H_SCALE ) ;
  *py = WIN_HEIGHT / 2 - (int)( z * V_SCALE ) ;
}

int redraw ( Display *d, Window w, GC gc, double theta, double phy ) {
  int x1, y1, x2, y2, i;
  
  XClearWindow ( d, w ) ;
  /* draw axis */
  if ( axis == True ) {
    trans ( 0, 0, 0, &x1, &y1, theta, phy ) ;
    trans ( 3, 0, 0, &x2, &y2, theta, phy ) ;
    XDrawLine ( d, w, gc, x1, y1, x2, y2 ) ;
    trans ( 0, 3, 0, &x2, &y2, theta, phy ) ;    
    XDrawLine ( d, w, gc, x1, y1, x2, y2 ) ;
    trans ( 0, 0, 3, &x2, &y2, theta, phy ) ;
    XDrawLine ( d, w, gc, x1, y1, x2, y2 ) ;
  }
  
  /* $BN)BN$rIA$/(B */
  for ( i = 0; i < NL; i++ ) {
    trans (
	   coord[ cnct[i].bgn ].x,
	   coord[ cnct[i].bgn ].y,
	   coord[ cnct[i].bgn ].z,
	   &x1, &y1, theta, phy ) ;
    trans (
	   coord[ cnct[i].end ].x,
	   coord[ cnct[i].end ].y,
	   coord[ cnct[i].end ].z,
	   &x2, &y2, theta, phy ) ;
    XDrawLine ( d, w, gc, x1, y1, x2, y2 ) ;
  }
  
  XFlush ( d ) ;
}

int main ( int argc, char *argv [] ) {
  Display *d;
  Window root, w;
  XEvent e;
  GC     gc;
  int i,  idx;
  double theta = 0;
  double phy = 0;
  FILE *fp;
  char line[1024], filnam[1024];
  int flg = 0;
  
  idx = 1;
  
  /* $B%3%^%s%I!&%i%$%s$+$i%Q%i%a!<%?$r<hF@(B */
  while ( idx < argc ) {
    if  ( strcmp ( argv[idx], "-a") == 0 )
      /*    $B:BI8<4IA2h%U%i%0(B*/ 
      axis = True;
    else
      /*$B%U%!%$%kL><hF@(B */
      strcpy ( filnam, argv[idx] ) ;
    idx++;
  }
  if ( strcmp (filnam, "") == 0 )
    {
      fprintf ( stderr, "error : not specified data file.\n") ;
    }
  
  /* $B:BI8$*$h$S7k@~%G!<%?$NFI$_9~$_(B */
  fp = fopen ( filnam, "r" ) ;
  i = 0;
  while ( feof (fp) == 0 ) {
    fgets ( line, 1024, fp ) ;

    if ( strlen (line) <= 1 ) {
      flg = 1;
      i = 0;
    }
    else {
      if ( flg == 0 ) {
	sscanf ( line, "%lg%lg%lg",
		 &coord[i].x, &coord[i].y, &coord[i].z ) ;  
	
      }
      else {
	sscanf ( line, "%d%d\n", &cnct[i].bgn, &cnct[i].end ) ;
      }
      i++;
    }
  }
  fclose (fp) ; 

  d = XOpenDisplay (NULL) ;
  root = RootWindow ( d, 0 ) ;
  w = XCreateSimpleWindow ( d, root, 25, 25, WIN_WIDTH, WIN_HEIGHT, 2,
			    BlackPixel (d, 0), WhitePixel (d,0)) ;
  /* $B%^%&%9!&%\%?%s!"%&%$%s%I%&I=<(!"%]%$%s%?0\F0$N(B
     $B%$%Y%s%H$rJaB*$9$k(B */
  XSelectInput ( d, w, ButtonPressMask | ExposureMask | KeyPressMask | ButtonMotionMask ) ;
  gc = XCreateGC ( d, w, 0, 0 ) ;
  XMapWindow ( d,w );
  
  while ( 1 ) {
    KeySym key;
    char string[10];
    int en;
    int x;
    XNextEvent ( d, &e );


    if ( e.type == MotionNotify ) {
      /* $B:8%I%i%C%0$r$7$?$H$-$N5sF0(B */
      if (e.xmotion.state == Button1Mask) {
	while (en == 0 ) {
	  /* $B&H$rJQ49$9$k!#(B*/
	  theta = theta - 0.1;
	  /* $B?7$7$$3QEY$GIA$-D>$7(B */
	  redraw ( d, w, gc, theta, phy ) ;
	  ++en;
	}
      }
      
      /* $B1&%I%i%C%0$r$7$?$H$-$N5sF0(B */
      if (e.xmotion.state == Button3Mask) {
	while (en == 0 ) {
	  /* $B&H$rJQ49$9$k!#(B*/
	  theta = theta + 0.1;
	  /* $B?7$7$$3QEY$GIA$-D>$7(B */
	  redraw ( d, w, gc, theta, phy ) ;
	  ++en;
	}
      }

      /* $BCf%I%i%C%0$r$7$?$H$-$N5sF0(B */
      if (e.xmotion.state == Button2Mask) {
	while (en == 0 ) {
	  /* $B&U$rJQ49$9$k!#(B*/
	  phy = phy + 0.1;
	  /* $B?7$7$$3QEY$GIA$-D>$7(B */
	  redraw ( d, w, gc, theta, phy ) ;
	  en = en +1;
	}
      }      
    }

    //if ( e.type == ButtonPress )
    /* $B%&%$%s%IFb$G%^%&%9!&%\%?%s$r2!$7$?$i%k!<%W$rC&$7$F=*N;(B */
    //  break;
    if ( e.type == Expose ) {
      /* $B%&%$%s%I$,I=$K$G$?>l9g$K$OIA$-D>$7(B */
      redraw ( d, w, gc, theta, phy ) ;
    } else
      if( e.type == KeyPress ){
	XLookupString((XKeyEvent *)&e,string,sizeof(string),&key,NULL);
	if(string[0] == '5'){
	  break;
	}	
	if(string[0] == 'q'){
	  break;
	}	
	
	if(string[0] == '6'){
	  while (en == 0 ) {
	    
	    for (x = 0; x <= 100; ++x){

	    /* $B&H$rJQ49$9$k!#(B*/
	    theta = theta + 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    
	    }
	    ++en;
	  }
	}
			
	if(string[0] == '4'){
	  while (en == 0 ) {

	    for (x = 0; x <= 100; ++x){

	    /* $B&H$rJQ49$9$k!#(B*/
	    theta = theta - 0.075;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    
	    }
	    ++en;
	  }
	}
	
	if(string[0] == '8'){
	  while (en == 0 ) {

	    for (x = 0; x <= 100; ++x){

	    /* $B&U$rJQ49$9$k!#(B*/
	    phy = phy + 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    en = en +1;
	  }
	}
	
	if(string[0] == '2'){
	  while (en == 0 ) {
	    
	    for (x = 0; x <= 100; ++x){

	    /* $B&U$rJQ49$9$k!#(B*/
	    phy = phy - 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    en = en +1;
	  }
	}
	
	if(string[0] == 'j'){
	  while (en == 0 ) {

	    for (x = 0; x <= 100; ++x){

	    /* $B&H$rJQ49$9$k!#(B*/
	    theta = theta + 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    ++en;
	  }
	}
			
	if(string[0] == 'h'){
	  while (en == 0 ) {
	    
	    for (x = 0; x <= 100; ++x){
	    
	    /* $B&H$rJQ49$9$k!#(B*/
	    theta = theta - 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    ++en;
	  }
	}
			
	if(string[0] == 'n'){
	  while (en == 0 ) {
	    
	    for (x = 0; x <= 100; ++x){

	    /* $B&U$rJQ49$9$k!#(B*/
	    phy = phy + 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    en = en +1;
	  }
	}
			
	if(string[0] == 'u'){
	  while (en == 0 ) {
	    
	    for (x = 0; x <= 100; ++x){

	    /* $B&U$rJQ49$9$k!#(B*/
	    phy = phy - 0.1;
	    /* $B?7$7$$3QEY$GIA$-D>$7(B */
	    redraw ( d, w, gc, theta, phy ) ;
	    }
	    en = en +1;
	  }
	}
      }
    en = 0;
  }
  XCloseDisplay (d) ;
}
